-- WhisperAction
-- by Zreptil

-- global saved variables
WAConfig = {};

-- constants
WA_ONLINE = ".*%[(.+)%]%S*"..string.sub(ERR_FRIEND_ONLINE_SS,20)
WA_OFFLINE = string.format(ERR_FRIEND_OFFLINE_S,"(.+)")
WA_NOPLAYER = string.format(ERR_CHAT_PLAYER_NOT_FOUND_S,"(.+)")
WA_PREFIX = "@WA_"
WA_SUFFIX = "@"

-- local variables
WAPlayer = nil;
evtOrgMsg = nil;
evtOrgFC = nil;
WASkipOrgEvt = false;
doDebug = false;
WATarget = nil;
WAFollowing = nul;
WARemoteState ={};
-- WAPhrases and WAPermissions is updated in WA_Initialize
-- with the values of WAConfig[x]["allow"]
WAPhrases =
  {
  ["requeststate"] = WA_PREFIX.."requeststate"..WA_SUFFIX;
  ["replystate"]   = WA_PREFIX.."replystate"..WA_SUFFIX;
  ["followabort"] = WA_PREFIX.."followabort"..WA_SUFFIX;
  ["followinit"] = WA_PREFIX.."followinit"..WA_SUFFIX;
  }
WAPermissions = {};
WARemotePlayerOff = false;

-------------------------------------------------------------------------
--
-- recreate the saved variables
--
-------------------------------------------------------------------------
function WA_Reset(bForce)
  local playerName = UnitName("player").." of "..GetCVar("realmName");

  if ( bForce or not WAConfig[playerName] ) then
    WAConfig[playerName] =
      {
      ["Version"] = WAVarVersion;
      ["commandPlayer"] = UnitName("player");
      ["invitePlayer"] = UnitName("player");
      ["remotePlayer"] = UnitName("player");
      ["minimappos"] = 275;
      ["allow"] =
        {
        ["follow"] = 1;
        ["trade"] = 1;
        ["relGhost"] = 1;
        ["revive"] = 1;
        ["group"] = 1;
        ["quest"] = 1;
        }
      }
  end
  WAPlayer = WAConfig[playerName];
end

-------------------------------------------------------------------------
--
-- executed when addon is loaded
--
-------------------------------------------------------------------------
function WA_OnLoad()
  this:RegisterEvent("VARIABLES_LOADED");
  this:RegisterEvent("TRADE_ACCEPT_UPDATE");
  this:RegisterEvent("TRADE_CLOSE");
  this:RegisterEvent("AUTOFOLLOW_BEGIN");
  this:RegisterEvent("AUTOFOLLOW_END");
  this:RegisterEvent("ADDON_ACTION_FORBIDDEN");
  this:RegisterEvent("ADDON_ACTION_BLOCKED");

  SlashCmdList["WHISPERACTION"] = function(msg) WA_SlashHandler(msg) end;
  SLASH_WHISPERACTION1 = "/whisperaction";
  SLASH_WHISPERACTION2 = "/wa";

  if ( FC_ChatFrame_OnEvent ) then
    evtOrgFC = FC_ChatFrame_OnEvent;
    FC_ChatFrame_OnEvent = WA_FCHandler;
  end

  evtOrgMsg = ChatFrame_OnEvent;
  ChatFrame_OnEvent = WA_MsgHandler;
end

-------------------------------------------------------------------------
--
-- for debugging purposes
--
-------------------------------------------------------------------------
function WA_Debug(msg)
  if ( doDebug ) and ( DEFAULT_CHAT_FRAME ) then
    DEFAULT_CHAT_FRAME:AddMessage("<WhisperActionDebug> "..msg, 1.0, 1.0, 0.0);
  end
end

-------------------------------------------------------------------------
--
-- initialization
--
-------------------------------------------------------------------------
function WA_Initialize()
  local playerName = UnitName("player").." of "..GetCVar("realmName");

  if      ( WAConfig[playerName] )
      and (   ( not WAConfig[playerName].Version )
           or ( WAConfig[playerName].Version ~= WAVarVersion )
          )
    then
    WA_Reset(true);
    message("Die Daten von WhisperAction\r\nwurden neu vorbelegt.");
  else
    WA_Reset(false);
  end


  WAPlayer = WAConfig[playerName];

  table.foreach(WAPlayer["allow"],
    function(k,v)
      WAPhrases[k]=WA_PREFIX..k..WA_SUFFIX;
      WAPermissions[k]=1;
      end);

  WAPermissions["trade"] = 0;
  WA_ChangeMiniMapPos(WAPlayer.minimappos);
  WA_ReplyState();
  return WAPlayer;
end

-------------------------------------------------------------------------
--
-- eventhandler
--
-------------------------------------------------------------------------
function WA_OnEvent(event)
  if ( event == "VARIABLES_LOADED" ) then
    local playerName = UnitName("player");
    if ( playerName ~= UNKNOWNBEING ) and ( playerName ~= "Unknown Entity" ) and ( WAPlayer == nil ) then
      WA_Chat("WhisperAction v"..WAVersion.." von Zreptil geladen.");
      WA_Initialize();
    end
  elseif ( event == "TRADE_ACCEPT_UPDATE" ) and ( arg1 == 1 ) then
    WAPermissions["trade"] = 1;
    WA_UpdateRC();
  elseif ( event == "TRADE_CLOSE" ) then
    WAPermissions["trade"] = 1;
    WA_UpdateRC();
  elseif ( event == "AUTOFOLLOW_BEGIN" ) then
    WAFollowing = arg1;
    if ( WAFollowing == WAPlayer.commandPlayer ) then
      WA_Whisper("followinit");
    end
  elseif ( event == "AUTOFOLLOW_END" ) then
    if ( WAFollowing == WAPlayer.commandPlayer ) then
      WA_Whisper("followabort");
    end
  elseif ( doDebug and event == "ADDON_ACTION_FORBIDDEN" ) then
    DEFAULT_CHAT_FRAME:AddMessage(arg1.." forbidden "..arg2);
  elseif ( doDebug and event == "ADDON_ACTION_BLOCKED" ) then
    DEFAULT_CHAT_FRAME:AddMessage(arg1.." blocked "..arg2);
  end
end

-------------------------------------------------------------------------
--
-- eventhandler for ForgottenChat
--
-------------------------------------------------------------------------
function WA_FCHandler(event)
  WASkipOrgEvt = true;
  WA_MsgHandler(event);
  WASkipOrgEvt = false;
  evtOrgFC(event);
end

-------------------------------------------------------------------------
--
-- eventhandler
--
-------------------------------------------------------------------------
function WA_MsgHandler(event)
  removeFromChat = 0;

  if ( event == "CHAT_MSG_WHISPER" or event == "CHAT_MSG_WHISPER_INFORM" ) then
    evt = "";
    params = "";
    if ( strsub(arg1,1,4) ~= WA_PREFIX ) then
      if ( not WASkipOrgEvt ) then
        evtOrgMsg(event);
      end
      return;
    end

    removeFromChat = 1;
    if ( arg2 ) and ( ( arg2 == WAPlayer.commandPlayer ) or ( arg2 == WAPlayer.remotePlayer ) ) then
      table.foreach(WAPhrases,function(k,v)if(strsub(arg1,1,strlen(v))==v)then evt=k; params=strsub(arg1,strlen(v)+1); end end)
      isCommander = ( arg2 == WAPlayer.commandPlayer );
      isRemote = ( arg2 == WAPlayer.remotePlayer );
      if ( evt ~= "" ) and ( event == "CHAT_MSG_WHISPER_INFORM" ) then
        return;
      end

      if ( evt ~= "" ) and ( WAPlayer.allow[evt] ~= nil ) and ( not WAPlayer.allow[evt] ) then
        return;
      end

      WA_ChangeTarget(arg2);
      if ( evt == "follow" ) and ( isCommander ) then
--        if ( CheckInteractDistance("target",4) ) then
          local msg = WAMessages["follow"];
          if ( UnitSex("target") == 0 ) then
            msg = WAMessages["followm"];
          elseif ( UnitSex("target") == 1 ) then
            msg = WAMessages["followf"];
          end

          WA_Chat(format(msg,arg2));
          FollowUnit(arg2);
--        else
--          WA_Chat(format(WAMessages["followtofar"],arg2));
--        end
      elseif ( evt == "relGhost" ) and ( isCommander ) then
        WA_Chat(format(WAMessages["relGhost"],arg2));
        RepopMe();
      elseif ( evt == "revive" ) and ( isCommander ) then
        WA_Chat(format(WAMessages["revive"],arg2));
        RepopMe();
        AcceptResurrect();
      elseif ( evt == "trade" ) and ( isCommander ) then
        if ( CheckInteractDistance("target",2) ) then
          WA_Chat(format(WAMessages["trade"],arg2));
          AcceptTrade();
        else
          WA_Chat(format(WAMessages["tradetofar"],arg2));
        end
      elseif ( evt == "group" ) and ( isCommander ) then
        AcceptGroup();
        if ( UnitInParty("target") ) then
          WA_Chat(format(WAMessages["group"],arg2));
        end
      elseif ( evt == "quest" ) and ( isCommander ) then
        AcceptQuest();
        WA_Chat(format(WAMessages["quest"],arg2));
      elseif ( evt == "requeststate" ) and ( isCommander ) then
        WA_ReplyState();
      elseif ( evt == "replystate" ) and ( isRemote ) then
        table.foreach(WAPlayer["allow"],function(k,v)WARemoteState[k]=v; end);
        WARemotePlayerOff = false;
        WA_UpdateRC();
      elseif ( evt == "followabort" ) and ( isRemote ) then
        WA_Chat(format(WAMessages["followabort"],WAPlayer.remotePlayer));
      elseif ( evt == "followinit" ) and ( isRemote ) then
        WA_Chat(format(WAMessages["followinit"],WAPlayer.remotePlayer));
      end
      WA_RestoreTarget();
    end
  elseif ( event == "CHAT_MSG_SYSTEM" or event == "CHAT_MSG" ) then
    if ( WA_CheckString(arg1,WAPlayer.invitePlayer,WA_ONLINE) ) then
      PlaySoundFile("Interface\\AddOns\\WhisperAction\\RemoteEnter.wav");
      InviteUnit(WAPlayer.invitePlayer);
    elseif ( WA_CheckString(arg1,WAPlayer.remotePlayer,WA_ONLINE) ) then
      WA_Whisper("requeststate");
      WARemotePlayerOff = false;
      WA_UpdateRC();
    elseif ( WA_CheckString(arg1,WAPlayer.remotePlayer,WA_NOPLAYER) ) then
      WA_Chat(format(WAMessages["noplayer"],WAPlayer.remotePlayer));
      WARemotePlayerOff = true;
      removeFromChat = 1;
      WA_UpdateRC();
    end
  end

  if ( removeFromChat == 0 ) and ( not WASkipOrgEvt ) then
    evtOrgMsg(event);
  end
end

-------------------------------------------------------------------------
--
-- ask remote player for stateinformation
--
-------------------------------------------------------------------------
function WA_ReplyState()
  WA_UpdateRC();
  answer = WAPhrases["replystate"];
  table.foreach(WAPlayer["allow"],function(k,v)answer=answer..v end);
  SendChatMessage(answer,"WHISPER",GetDefaultLanguage("player"),WAPlayer.commandPlayer);
end

-------------------------------------------------------------------------
--
-- check if string contains online message for given player
--
-------------------------------------------------------------------------
function WA_CheckString(text,search,regex)
  if ( text == nil ) then
    return nil;
  end

  local _, _, found = string.find(text,regex);
  return ( found == search );
end

-------------------------------------------------------------------------
--
-- change the target to the player with the given name
--
-------------------------------------------------------------------------
function WA_ChangeTarget(name)
--  WATarget = UnitName("target");
--  TargetByName(name);
end

-------------------------------------------------------------------------
--
-- change the target to the player with the given name
--
-------------------------------------------------------------------------
function WA_RestoreTarget()
--  if ( WATarget == nil ) then
--    ClearTarget();
--  else
--    TargetByName(WATarget);
--  end
end

-------------------------------------------------------------------------
--
-- executed when a slashcommand is entered
--
-------------------------------------------------------------------------
function WA_SlashHandler(msg)
  if ( strlen(msg) == 0 ) then
    WA_Chat(WAMessages["slashhelp"]);
    return;
  end

  if ( strsub(msg,1,5) == "debug" ) then
    doDebug = not doDebug;
  elseif ( msg == "reset" ) then
    WA_Reset(true);
  end
end

-------------------------------------------------------------------------
--
-- searches for a mount in the bags returns the first mount found
--
-------------------------------------------------------------------------
function WA_FindMountSlot()
  for iBag=0, 4 do
    iCount = GetContainerNumSlots(iBag);
    for iSlot=0, iCount do
      local texture = GetContainerItemInfo(iBag,iSlot);
      if ( texture ) and ( strfind(texture,"_Mount_") ) then
        return iBag,iSlot;
      end
    end
  end

  return nil,nil;
end

-------------------------------------------------------------------------
--
-- searches for a mount in the buffs
--
-------------------------------------------------------------------------
function WA_FindMountBuff()
  i = 0;
  while GetPlayerBuffTexture(i) ~= nil do
    if ( strfind(GetPlayerBuffTexture(i),"_Mount_") ) then
      return i;
    end
    i = i + 1;
  end

  return nil;
end

-------------------------------------------------------------------------
--
-- searches for a mount in the actionbars
--
-------------------------------------------------------------------------
function WA_FindMountAction()
  for iSlot=0,256 do
    if ( GetActionTexture(iSlot) ) and ( strfind(GetActionTexture(iSlot),"_Mount_") ) then
      return iSlot;
    end
  end

  return nil;
end

-------------------------------------------------------------------------
--
-- display a message in the normal chatwindow
--
-------------------------------------------------------------------------
function WA_Chat(msg)
  if( DEFAULT_CHAT_FRAME ) then
    DEFAULT_CHAT_FRAME:AddMessage("<WhisperAction> "..msg,0.0,1.0,0.0);
  end
end

-------------------------------------------------------------------------
--
-- eventhandler for the button at the minimap
--
-------------------------------------------------------------------------
function WA_FrameToggle(button)
  local frame

  if ( button == "RightButton" ) then
    if ( getglobal("WARemoteControl"):IsVisible() ) then
      getglobal("WARemoteControl"):Hide();
    end
    frame = getglobal("WACfgFrame");
  elseif ( button == "LeftButton" ) then
    if ( getglobal("WACfgFrame"):IsVisible() ) then
      return;
    end
    frame = getglobal("WARemoteControl");
  end

  if ( frame:IsVisible() ) then
    frame:Hide();
  else
    frame:Show();
    WA_LoadConfig();
  end
end

-------------------------------------------------------------------------
--
-- load the value in the configdialog
--
-------------------------------------------------------------------------
function WA_LoadConfig()
  getglobal("editWACCommandPlayer"):SetText(WAPlayer.commandPlayer);
  getglobal("editWACInvitePlayer"):SetText(WAPlayer.invitePlayer);
  getglobal("editWACRemotePlayer"):SetText(WAPlayer.remotePlayer);

  table.foreach(WAPlayer["allow"],
  function(k,v)
    local ctrl = "chkWAC"..WA_FirstLetterBig(k);
    getglobal(ctrl):SetChecked(v);
  end);
end

-------------------------------------------------------------------------
--
-- save values from configdialog
--
-------------------------------------------------------------------------
function WA_SaveConfig()
  WAPlayer.commandPlayer = WA_FirstLetterBig(getglobal("editWACCommandPlayer"):GetText());
  WAPlayer.invitePlayer = WA_FirstLetterBig(getglobal("editWACInvitePlayer"):GetText());
  WAPlayer.remotePlayer = WA_FirstLetterBig(getglobal("editWACRemotePlayer"):GetText());

  table.foreach(WAPlayer["allow"],
  function(k,v)
    local ctrl = "chkWAC"..WA_FirstLetterBig(k);
    if ( getglobal(ctrl):GetChecked() ) then
      WAPlayer.allow[k] = 1;
    else
      WAPlayer.allow[k] = 0;
    end
  end);

  WA_ReplyState();
end

-------------------------------------------------------------------------
--
-- make first letter of string uppercase
--
-------------------------------------------------------------------------
function WA_FirstLetterBig(text)
  return strupper(strsub(text,1,1))..strsub(text,2);
end

-------------------------------------------------------------------------
--
-- use target as commandplayer
--
-------------------------------------------------------------------------
function WA_UseTargetAsCommand()
  if ( UnitIsPlayer("target") ) then
    getglobal("editWACCommandPlayer"):SetText(UnitName("target"));
  else
    message("Das Ziel muss ein Spieler sein.");
  end
end

-------------------------------------------------------------------------
--
-- use target as inviteplayer
--
-------------------------------------------------------------------------
function WA_UseTargetAsInvite()
  if ( UnitIsPlayer("target") ) then
    getglobal("editWACInvitePlayer"):SetText(UnitName("target"));
  else
    message("Das Ziel muss ein Spieler sein.");
  end
end

-------------------------------------------------------------------------
--
-- use target as remoteplayer
--
-------------------------------------------------------------------------
function WA_UseTargetAsRemote()
  if ( UnitIsPlayer("target") ) then
    getglobal("editWACRemotePlayer"):SetText(UnitName("target"));
  else
    message("Das Ziel muss ein Spieler sein.");
  end
end

-------------------------------------------------------------------------
--
-- send a whisper to the remoteplayer
--
-------------------------------------------------------------------------
function WA_Whisper(phrase)
  SendChatMessage(WAPhrases[phrase],"WHISPER",GetDefaultLanguage("player"),WAPlayer.remotePlayer);
  if ( WASent[phrase] ) then
    WA_Chat(format(WASent[phrase],WAPlayer.remotePlayer));
  end
end

-------------------------------------------------------------------------
--
-- displays a tooltip
--
-------------------------------------------------------------------------
function WA_ShowTooltip(control)
  GameTooltip:SetOwner(this,"ANCHOR_LEFT");
  GameTooltip:SetBackdropColor(0.0,0.0,0.0);
  GameTooltip:SetText(format(WAToolTips[control:GetName()],WAPlayer.remotePlayer),0.8,0.8,0.0);
end

-------------------------------------------------------------------------
--
-- reads a text from the config and sets it to the control
--
-------------------------------------------------------------------------
function WA_SetText(control)
  if ( not WAPlayer ) then
    return;
  end

  WANames["WARTitle"] = WAPlayer.remotePlayer;
  getglobal(control):SetText(WANames[control]);
end

-------------------------------------------------------------------------
--
-- change the position of the minimap, called by the changeevent
-- of the slider
--
-------------------------------------------------------------------------
function WA_ChangeMiniMapPos(value)
  btn = getglobal("WAMinimapButton");
  if ( btn == nil ) then
    return;
  end

  r = 80;
  x = cos(value)*r;
  y = sin(value)*r;
  btn:SetPoint("CENTER","Minimap","CENTER",x,y);

  if ( WAPlayer ~= nil ) then
    WAPlayer.minimappos = value;
  end
end

-------------------------------------------------------------------------
--
-- fill the controls on the configdialog
--
-------------------------------------------------------------------------
function WA_FillConfig()
  WA_SetText("WACTitle");
  WA_SetText("WACSave");
  WA_SetText("WACCancel");
  WA_SetText("btnWACCommandPlayer");
  WA_SetText("btnWACInvitePlayer");
  WA_SetText("btnWACRemotePlayer");
  WA_SetText("textWACCommand");
  WA_SetText("textWACInvite");
  WA_SetText("textWACRemote");

  table.foreach(WAPlayer["allow"],
    function(k,v)
      ctrl = "textWAC"..WA_FirstLetterBig(k);
      WA_SetText(ctrl);
    end);

  WA_SetText("textWACMinimap");
  WA_SetText("textWACMinimapMin");
  WA_SetText("textWACMinimapMax");
  getglobal("slideWACMinimap"):SetValue(WAPlayer.minimappos);
end

-------------------------------------------------------------------------
--
-- update the controls on the remotecontrol using information requested
-- from the remote player
--
-------------------------------------------------------------------------
function WA_UpdateRC()
  WA_SetText("WARTitle");
  WA_SetText("WARClose");

  if ( WAPlayer == nil ) then
    return;
  end

  table.foreach(WAPlayer["allow"],
    function(k,v)
      ctrl = "WAR"..WA_FirstLetterBig(k);
      WA_SetText(ctrl);

--      if   ( WARemoteState[k] == 0 )
--        or ( WAPermissions[k] == 0 )
--        or ( WARemotePlayerOff )
--      then
--        getglobal(ctrl):Disable();
--      else
--        getglobal(ctrl):Enable();
--      end
    end);
end


